#pragma once
#ifndef PARSODUS_BACKENDMANAGER_H
#define PARSODUS_BACKENDMANAGER_H

#include <memory>
#include <string>
#include <vector>

#include "Parsodus/backend.h"
#include "Parsodus/lrtables/generator.h"
#include "Parsodus/lrtables/LR0Itemset.h"
#include "Parsodus/lrtables/SLR1Itemset.h"
#include "Parsodus/util/parserType.h"

namespace pds {
    /**
     * A manager for backends
     * Aggregates and allow to search for backends that can process a specific language
     */
    class BackendManager {
        public:
            /**
             * Add a backend to the list of registered backends
             *
             * @param backend The backend to register
             */
            void registerBackend(std::unique_ptr<Backend> backend);

            template<template<class > class T>
            void registerLR() {
                registerBackend(std::make_unique<T<lr::Generator<lr::LR0Itemset>>>(util::ParserType::LR_0)); 
                registerBackend(std::make_unique<T<lr::Generator<lr::SLR1Itemset>>>(util::ParserType::SLR_1)); 
            }
            
            /**
             * Get a backend that can process the given language
             * The manager retains ownership of the returned pointer
             *
             * @param lang The language the backend should be able to process
             * @param parserType the type of parser it should be able to produce (e.g. LALR_1, LR_0, etc..)
             * @returns A pointer to a Backend if it can find one, nullptr otherwise
             */
            Backend* findBackend(std::string lang, util::ParserType parserType);

        private:
            std::vector<std::unique_ptr<Backend> > m_backends; ///< The list of registered backends
    };
}

#endif //PARSODUS_BACKENDMANAGER_H

