#include <iostream>
#include <fstream>
#include "optparse.h"
#include "Parsodus/inputparser.h"
#include "Parsodus/backendmanager.h"
#include "Parsodus/lrtables/generator.h"
#include "Parsodus/lrtables/SLR1Itemset.h"
#include "Parsodus/backends/cppLR.h"
#include "Parsodus/util/parserType.h"

int main(int argc, char** argv) {

    optparse::OptionParser parser = optparse::OptionParser().description("Parsodus").usage("Parsodus [-d <outputdir>] [-l <language>] [-n <lexername>] <inputfile.pds>");
    parser.add_help_option(true);
    parser.version("%prog 1.0");
    parser.add_option("-d", "--outputdir").dest("outputdir").help("Output the generated files to this directory\n[default: .]").metavar("<directory>").set_default(".");
    parser.add_option("-l", "--lang", "--language").dest("language").help("The programming language to generate source files for\n[default: c++]").metavar("<language>").set_default("c++");
    parser.add_option("-n", "--name").dest("lexername").help("Use this name for the generated parser, the default is based on the input file name").metavar("<parsername>");
    optparse::Values options = parser.parse_args(argc, argv);
    std::vector<std::string> args = parser.args();

    if (args.size() != 1) {
        parser.print_usage(std::cerr);
        return 1;
    }

    std::ifstream infile(args[0]);
    if (!infile.good()) {
        std::cerr << "Could not open file '" << args[0] << "' for reading" << std::endl;
        return 1;
    }

    auto config = pds::InputParser::parseInput(infile);
   
    pds::BackendManager b;
    b.registerLR<pds::backends::CppLRBackend>();
    pds::Backend* back = b.findBackend("c++", pds::util::ParserType::LR_0);
    std::cout << back->getName() << std::endl;
    
    // Reporting what the inputparser found, to be removed...
    std::cout << "Start: " << config.grammar.start << std::endl;
    for(auto a: config.grammar.terminals)
        std::cout << "Terminal: " << a << std::endl;
    for(auto a: config.grammar.variables)
        std::cout << "Variable: " << a << std::endl;
    std::cout << "Rules: " << std::endl;
    for(auto a: config.grammar.rules) {
        std::cout << "\t" << a->head << " -> ";
        for(auto c: a->tail) {
            std::cout << c << " ";
        }
        std::cout << std::endl;
    }

    std::vector<std::string> names = {"ERROR", "SHIFT", "REDUCE", "ACCEPT"};
    pds::lr::Generator<pds::lr::SLR1Itemset> g(config.grammar);
    auto tbl = g.generate();
    for (std::size_t i = 0; i < tbl.act.size(); i++) {
        std::cout << "State " << i << std::endl;
        std::cout << " Action:" << std::endl;
        for (auto& p : tbl.act[i]) {
            std::cout << "  " << p.first << ": " << names[static_cast<int>(p.second.first)] << " " << p.second.second << std::endl;
        }
        std::cout << " Goto:" << std::endl;
        for (auto& p : tbl.goto_[i]) {
            std::cout << "  " << p.first << ": " << p.second << std::endl;;
        }
    }
}
